#!/usr/bin/python3

import argparse
import os
import subprocess


def parse_arguments():
    parser = argparse.ArgumentParser(description='Modify GSIMerging2.hh based on merge_config.txt and command-line arguments.')
    parser.add_argument("-F", "--sim_output_file", required=True, help="Name of the simulation output file")
    parser.add_argument("-D", "--sim_output_dir", required=True, help="Name of the simulation output directory")
    parser.add_argument("-R", "--change_RunToTreat", required=True, help="1 To change RunToTreat")
    return parser.parse_args()

def parse_config(config_path):
    config_vars = {}
    with open(config_path, 'r') as config_file:
        for line in config_file:
            if '=' in line:
                var, value = line.split('=')
                config_vars[var.strip()] = value.strip()
    return config_vars

def modify_header_file(header_path, config_vars, sim_output_dir, sim_output_file):
    temp_file_path = header_path + '.tmp'
    with open(header_path, 'r') as header_file, open(temp_file_path, 'w') as temp_file:
        for line in header_file:
            modified = False
            for var, value in config_vars.items():
                if f'const double {var} ' in line:
                    temp_file.write(f'const double {var} = {value};  // Modified by merge_simu script\n')
                    modified = True
                    break
            if 'const std::string FileName' in line:
                print(f'{sim_output_dir}/{sim_output_file}')
                temp_file.write(f'const std::string FileName = "{sim_output_dir}/{sim_output_file}"; \n')
                modified = True
            if not modified:
                temp_file.write(line)
    os.replace(temp_file_path, header_path)

def replace_line_in_file(file_path, identifier, new_line):
    with open(file_path, 'r') as file:
        lines = file.readlines()

    with open(file_path, 'w') as file:
        write_next = False
        for line in lines:
            if write_next:
                file.write(new_line + '\n')
                write_next = False
            else:
                file.write(line)
            if identifier in line:
                write_next = True


def run_root_script(cxx_path):
    # Path to thisroot.sh (adjust as necessary for your ROOT installation)
    bash = "~/.bashrc"
    command = f"cd Scripts && source {bash} && root -l  -q {cxx_path}++"

    try:
        # Execute the command in bash to allow 'source' to work
        subprocess.run(command, shell=True, executable="/bin/bash", check=True)
        print("ROOT script executed successfully.")
    except subprocess.CalledProcessError as e :
        print(f"Failed to execute ROOT script: {e}")


def main():
    args = parse_arguments()


    nptool_path = os.getenv('NPTOOL')
    sim_dir = nptool_path + "/Outputs/Simulation/GSI"
    config_path ="DataConfig/merge_config.txt"
    header_path ="Scripts/GSIMerging3.hh"
    cxx_path = "GSIMerging3.cc"

    config_vars = parse_config(config_path)
    modify_header_file(header_path, config_vars, sim_dir+'/'+args.sim_output_dir, args.sim_output_file)
    run_root_script(cxx_path)

    sim_dir = sim_dir+'/'+args.sim_output_dir
    sim_file = args.sim_output_file
    if args.change_RunToTreat :
        new_path = f"{sim_dir}/{sim_file}_EDIN_tau" + str(config_vars['coinc_time'])
        new_path += "_T" +str(config_vars['t_max']) + "_TADC" + str(config_vars['T_ADC']) + ".root"

        replace_line_in_file('RunToTreat.txt', 'RootFileName', f'\t{new_path}')
        replace_line_in_file('RunToTreat.txt', 'TTreeName', '\tSimulatedTree')

if __name__ == "__main__":
    main()
